/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.office.filter.ooxml.xlsx.tools;

import java.util.Iterator;
import java.util.List;
import java.util.Map;
import org.docx4j.IndexedNodeList;
import org.docx4j.dml.CTBlipFillProperties;
import org.docx4j.dml.CTGroupShapeProperties;
import org.docx4j.dml.CTNonVisualDrawingProps;
import org.docx4j.dml.CTNonVisualPictureProperties;
import org.docx4j.dml.CTPoint2D;
import org.docx4j.dml.CTPositiveSize2D;
import org.docx4j.dml.CTPresetGeometry2D;
import org.docx4j.dml.CTShapeProperties;
import org.docx4j.dml.STShapeType;
import org.docx4j.dml.spreadsheetdrawing.CTAbsoluteAnchor;
import org.docx4j.dml.spreadsheetdrawing.CTConnector;
import org.docx4j.dml.spreadsheetdrawing.CTConnectorNonVisual;
import org.docx4j.dml.spreadsheetdrawing.CTGraphicalObjectFrame;
import org.docx4j.dml.spreadsheetdrawing.CTGroupShape;
import org.docx4j.dml.spreadsheetdrawing.CTGroupShapeNonVisual;
import org.docx4j.dml.spreadsheetdrawing.CTMarker;
import org.docx4j.dml.spreadsheetdrawing.CTOneCellAnchor;
import org.docx4j.dml.spreadsheetdrawing.CTPicture;
import org.docx4j.dml.spreadsheetdrawing.CTShape;
import org.docx4j.dml.spreadsheetdrawing.CTShapeNonVisual;
import org.docx4j.dml.spreadsheetdrawing.CTTwoCellAnchor;
import org.docx4j.dml.spreadsheetdrawing.ICellAnchor;
import org.docx4j.dml.spreadsheetdrawing.ObjectFactory;
import org.docx4j.dml.spreadsheetdrawing.STEditAs;
import org.docx4j.jaxb.Context;
import org.docx4j.openpackaging.exceptions.InvalidFormatException;
import org.docx4j.openpackaging.exceptions.PartUnrecognisedException;
import org.docx4j.openpackaging.parts.Part;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import com.openexchange.office.filter.api.FilterException;
import com.openexchange.office.filter.api.FilterException.ErrorCode;
import com.openexchange.office.filter.api.OCKey;
import com.openexchange.office.filter.api.OCValue;
import com.openexchange.office.filter.ooxml.OperationDocument;
import com.openexchange.office.filter.ooxml.drawingml.ChartWrapper;
import com.openexchange.office.filter.ooxml.drawingml.DMLGraphic;
import com.openexchange.office.filter.ooxml.drawingml.DMLHelper;
import com.openexchange.office.filter.ooxml.tools.Commons;
import com.openexchange.office.filter.ooxml.xlsx.XlsxOperationDocument;
import com.openexchange.office.imagemgr.DocumentImageHelper;

public class Drawings {

	private final static Logger log = LoggerFactory.getLogger(Drawings.class);

	public static void createAnchorOperations(ICellAnchor anchor, JSONObject attrs)
	    throws JSONException {

	    final JSONObject drawingProperties = new JSONObject();
        if(anchor instanceof CTAbsoluteAnchor) {
            drawingProperties.put(OCKey.ANCHOR_TYPE.value(), "absolute");
            CTPoint2D position = ((CTAbsoluteAnchor) anchor).getPos(false);
            CTPositiveSize2D size = ((CTAbsoluteAnchor) anchor).getExt(false);
            if(position!=null) {
                drawingProperties.put(OCKey.LEFT.value(), Commons.coordinateTo100TH_MM(position.getX()));
                drawingProperties.put(OCKey.TOP.value(), Commons.coordinateTo100TH_MM(position.getY()));
            }
            if(size!=null) {
                drawingProperties.put(OCKey.WIDTH.value(), Commons.coordinateTo100TH_MM(size.getCx()));
                drawingProperties.put(OCKey.HEIGHT.value(), Commons.coordinateTo100TH_MM(size.getCy()));
            }
        }
        else if(anchor instanceof CTTwoCellAnchor) {
            CTMarker from = ((CTTwoCellAnchor) anchor).getFrom(false);
            CTMarker to = ((CTTwoCellAnchor) anchor).getTo(false);
            if(from!=null) {
                drawingProperties.put(OCKey.START_COL.value(), from.getCol());
                drawingProperties.put(OCKey.START_COL_OFFSET.value(), Commons.coordinateTo100TH_MM(from.getColOff()));
                drawingProperties.put(OCKey.START_ROW.value(), from.getRow());
                drawingProperties.put(OCKey.START_ROW_OFFSET.value(), Commons.coordinateTo100TH_MM(from.getRowOff()));
            }
            if(to!=null) {
                drawingProperties.put(OCKey.END_COL.value(), to.getCol());
                drawingProperties.put(OCKey.END_COL_OFFSET.value(), Commons.coordinateTo100TH_MM(to.getColOff()));
                drawingProperties.put(OCKey.END_ROW.value(), to.getRow());
                drawingProperties.put(OCKey.END_ROW_OFFSET.value(), Commons.coordinateTo100TH_MM(to.getRowOff()));
            }
            if(((CTTwoCellAnchor) anchor).getEditAs()==STEditAs.ABSOLUTE) {
                drawingProperties.put(OCKey.ANCHOR_TYPE.value(), "twoCellAsAbsolute");
            }
            else if (((CTTwoCellAnchor) anchor).getEditAs()==STEditAs.ONE_CELL) {
                drawingProperties.put(OCKey.ANCHOR_TYPE.value(), "twoCellAsOneCell");
            }
            else {
                drawingProperties.put(OCKey.ANCHOR_TYPE.value(), "twoCell");
            }
        }
        else if(anchor instanceof CTOneCellAnchor) {
            drawingProperties.put(OCKey.ANCHOR_TYPE.value(), "oneCell");
            CTMarker from = ((CTOneCellAnchor) anchor).getFrom(false);
            CTPositiveSize2D size = ((CTOneCellAnchor) anchor).getExt(false);
            if(from!=null) {
                drawingProperties.put(OCKey.START_COL.value(), from.getCol());
                drawingProperties.put(OCKey.START_COL_OFFSET.value(), Commons.coordinateTo100TH_MM(from.getColOff()));
                drawingProperties.put(OCKey.START_ROW.value(), from.getRow());
                drawingProperties.put(OCKey.START_ROW_OFFSET.value(), Commons.coordinateTo100TH_MM(from.getRowOff()));
            }
            if(size!=null) {
                drawingProperties.put(OCKey.WIDTH.value(), Commons.coordinateTo100TH_MM(size.getCx()));
                drawingProperties.put(OCKey.HEIGHT.value(), Commons.coordinateTo100TH_MM(size.getCy()));
            }
        }
        else {
            throw new FilterException("XLSX import: unknown anchor type for drawing.", ErrorCode.CRITICAL_ERROR);
        }
        attrs.put(OCKey.DRAWING.value(), drawingProperties);
	}

    public static void insertDrawingOperation(JSONArray operationsArray, List<Integer> drawingPosition, String drawingType, JSONObject attrs)
        throws JSONException {

        final JSONObject insertDrawingOperation = new JSONObject(4);
        insertDrawingOperation.put(OCKey.NAME.value(), OCValue.INSERT_DRAWING.value());
        insertDrawingOperation.put(OCKey.START.value(), drawingPosition);
        insertDrawingOperation.put(OCKey.TYPE.value(), drawingType);
        insertDrawingOperation.put(OCKey.ATTRS.value(), attrs);
        operationsArray.put(insertDrawingOperation);
    }

    public static ICellAnchor applyAnchorAttributes(int drawingIndex, org.docx4j.openpackaging.parts.DrawingML.Drawing drawingsPart, JSONObject attrs)
        throws Exception {

        final org.docx4j.dml.spreadsheetdrawing.CTDrawing drawings = drawingsPart.getJaxbElement();
        final IndexedNodeList<Object> anchors = drawings.getContent();
        if (drawingIndex>=anchors.size()) {
            throw new RuntimeException("xlsx::applyDrawingOperation: drawing not available");
        }
        final JSONObject drawingProperties = attrs.optJSONObject(OCKey.DRAWING.value());
        if(drawingProperties==null) {
            return (ICellAnchor)anchors.get(drawingIndex);
        }
        final ICellAnchor cellAnchor = setDrawingAnchor(drawingProperties, (ICellAnchor)anchors.get(drawingIndex));
        anchors.set(drawingIndex, cellAnchor);
        return cellAnchor;
    }

    public static void applyDrawingAttributes(OperationDocument operationDocument, ICellAnchor anchor, Part part, JSONObject attrs)
    	throws JSONException, InvalidFormatException, PartUnrecognisedException {

        if(anchor.getPic()!=null) {
            DMLHelper.applyNonVisualDrawingProperties(anchor.getPic(), attrs.optJSONObject(OCKey.DRAWING.value()));
            DMLHelper.applyNonVisualDrawingShapeProperties(anchor.getPic(), attrs.optJSONObject(OCKey.DRAWING.value()));
    	    anchor.getPic().setSpPr(DMLHelper.applyShapePropertiesFromJson(anchor.getPic().getSpPr(), attrs, operationDocument, part, true));
        }
        else if(anchor.getGraphicFrame()!=null) {
            DMLHelper.applyNonVisualDrawingProperties(anchor.getGraphicFrame(), attrs.optJSONObject(OCKey.DRAWING.value()));
        	DMLHelper.applyNonVisualDrawingShapeProperties(anchor.getGraphicFrame(), attrs.optJSONObject(OCKey.DRAWING.value()));
        } else if (anchor.getAlternateContent() != null) {
            // System.out.println("Drawings.applyDrawingAttributes() " + anchor.getAlternateContent());
        }
    }

    public static void insertDrawing(XlsxOperationDocument operationDocument, int drawingIndex, org.docx4j.openpackaging.parts.DrawingML.Drawing drawingPart, String drawingType, JSONObject attrs)
        throws Exception {

        ICellAnchor anchor = Context.getDmlSpreadsheetDrawingObjectFactory().createCTTwoCellAnchor();
        anchor.setClientData(Context.getDmlSpreadsheetDrawingObjectFactory().createCTAnchorClientData());
        drawingPart.getJaxbElement().getContent().add(drawingIndex, anchor);

        anchor = applyAnchorAttributes(drawingIndex, drawingPart, attrs);

        // we have to create a valid drawing otherwise excel wont open... at least we are creating an default empty rect shape
        final JSONObject imageAttrs = attrs.optJSONObject(OCKey.IMAGE.value());
        if(drawingType.equals("image")) {
            Object imageUrl = null;
            if(imageAttrs!=null) {
                imageUrl = DocumentImageHelper.getImageUrlFromImageData(operationDocument.getResourceManager(), imageAttrs.optString(OCKey.IMAGE_DATA.value()), "xl/media/");
                if(imageUrl==null) {
                    imageUrl = imageAttrs.opt(OCKey.IMAGE_URL.value());
                }
            }
            if(imageUrl instanceof String) {
                anchor.setPic(createImage(operationDocument, drawingPart, (String)imageUrl));
            }
            else {
                anchor.setSp(createDummyShape());
            }
        }
        else if(drawingType.equals("chart")) {
            anchor.setGraphicFrame(createChart(operationDocument, drawingPart, attrs.optJSONObject(OCKey.CHART.value())));
        }
        else {
            anchor.setSp(createDummyShape());
        }
        applyDrawingAttributes(operationDocument, anchor, drawingPart, attrs);
    }

    public static CTGraphicalObjectFrame createChart(XlsxOperationDocument operationDocument, Part drawingPart, JSONObject chartAttrs)
        throws Exception {

        final CTGraphicalObjectFrame graphicalObjectFrame = Context.getDmlSpreadsheetDrawingObjectFactory().createCTGraphicalObjectFrame();
        graphicalObjectFrame.getNonVisualDrawingShapeProperties(true);
        final CTNonVisualDrawingProps nonVisualDrawingProperties = graphicalObjectFrame.getNonVisualDrawingProperties(true);
        nonVisualDrawingProperties.setName("chart");
        DMLHelper.createTransform2D(graphicalObjectFrame.getXfrm(true), 0, 0, 500, 500);
        graphicalObjectFrame.setGraphic(DMLGraphic.createDrawingMLChart(operationDocument, drawingPart, "/xl/charts/", chartAttrs));
        return graphicalObjectFrame;
    }

    public static CTPicture createImage(XlsxOperationDocument operationDocument, Part drawingPart, String imageUrl)
        throws Exception {

        final CTPicture picture = Context.getDmlSpreadsheetDrawingObjectFactory().createCTPicture();

        // general shape properties
        final CTShapeProperties shapeProperties = Context.getDmlObjectFactory().createCTShapeProperties();
        picture.setSpPr(shapeProperties);
        DMLHelper.createTransform2D(picture.getXfrm(true), 0, 0, 500, 500);
        shapeProperties.setPrstGeom(DMLHelper.createPresetGeometry2D(STShapeType.RECT));
        picture.getNonVisualDrawingShapeProperties(true);
        final CTNonVisualDrawingProps nonVisualDrawingProperties = picture.getNonVisualDrawingProperties(true);
        nonVisualDrawingProperties.setName("image");

        // picture
        final CTBlipFillProperties blipFillProperties = Context.getDmlObjectFactory().createCTBlipFillProperties();
        blipFillProperties.setBlip(DMLHelper.createCTBlip(operationDocument, drawingPart, imageUrl));
        blipFillProperties.setStretch(Context.getDmlObjectFactory().createCTStretchInfoProperties());
        picture.setBlipFill(blipFillProperties);
        return picture;
    }

    private static CTShape createDummyShape() {

        final CTShape shape = Context.getDmlSpreadsheetDrawingObjectFactory().createCTShape();

        // general shape Properties
        final CTShapeProperties shapeProperties = Context.getDmlObjectFactory().createCTShapeProperties();
        shape.setSpPr(shapeProperties);
        DMLHelper.createTransform2D(shape.getXfrm(true), 0, 0, 500, 500);
        shape.setNvSpPr(createShapeNonVisual("shape"));

        // shape properties (preset)
        shapeProperties.setPrstGeom(DMLHelper.createPresetGeometry2D(STShapeType.RECT));
        return shape;
    }

    private static CTShapeNonVisual createShapeNonVisual(String name) {
        final CTShapeNonVisual shapeNonVisual = Context.getDmlSpreadsheetDrawingObjectFactory().createCTShapeNonVisual();
        shapeNonVisual.setCNvSpPr(Context.getDmlObjectFactory().createCTNonVisualDrawingShapeProps());
        final CTNonVisualDrawingProps nonVisualDrawingProps = Context.getDmlObjectFactory().createCTNonVisualDrawingProps();
        nonVisualDrawingProps.setName(name);
        shapeNonVisual.setCNvPr(nonVisualDrawingProps);
        return shapeNonVisual;
    }

    private static CTGroupShapeNonVisual createGrpShapeNonVisual(String name) {
        final CTGroupShapeNonVisual shapeNonVisual = Context.getDmlSpreadsheetDrawingObjectFactory().createCTGroupShapeNonVisual();
        shapeNonVisual.setCNvGrpSpPr(Context.getDmlObjectFactory().createCTNonVisualGroupDrawingShapeProps());
        final CTNonVisualDrawingProps nonVisualDrawingProps = Context.getDmlObjectFactory().createCTNonVisualDrawingProps();
        nonVisualDrawingProps.setName(name);
        shapeNonVisual.setCNvPr(nonVisualDrawingProps);
        return shapeNonVisual;
    }

    private static CTConnectorNonVisual createConnectorShapeNonVisual(String name) {
        final CTConnectorNonVisual nvSpPr = Context.getDmlSpreadsheetDrawingObjectFactory().createCTConnectorNonVisual();
        nvSpPr.setCNvCxnSpPr(Context.getDmlObjectFactory().createCTNonVisualConnectorProperties());
        final CTNonVisualDrawingProps nonVisualDrawingProps = Context.getDmlObjectFactory().createCTNonVisualDrawingProps();
        nonVisualDrawingProps.setName(name);
        nvSpPr.setCNvPr(nonVisualDrawingProps);
        return nvSpPr;
    }

    private static CTMarker getFrom(ICellAnchor cellAnchor) {
        if(cellAnchor instanceof CTTwoCellAnchor) {
            return ((CTTwoCellAnchor)cellAnchor).getFrom(true);
        }
        else if(cellAnchor instanceof CTOneCellAnchor) {
            return ((CTOneCellAnchor)cellAnchor).getFrom(true);
        }
        log.warn("xlsx export:getFrom... wrong anchor type used");
        return new CTMarker();
    }

    private static CTMarker getTo(ICellAnchor cellAnchor) {
        if(cellAnchor instanceof CTTwoCellAnchor) {
            return ((CTTwoCellAnchor)cellAnchor).getTo(true);
        }
        log.warn("xlsx export:getTo... wrong anchor type used");
        return new CTMarker();
    }

    private static CTPositiveSize2D getExt(ICellAnchor cellAnchor) {
        if(cellAnchor instanceof CTOneCellAnchor) {
            return ((CTOneCellAnchor)cellAnchor).getExt(true);
        }
        else if(cellAnchor instanceof CTAbsoluteAnchor) {
            return ((CTAbsoluteAnchor)cellAnchor).getExt(true);
        }
        log.warn("xlsx export:getExt... wrong anchor type used");
        return new CTPositiveSize2D();
    }

    private static CTPoint2D getPos(ICellAnchor cellAnchor) {
        if(cellAnchor instanceof CTAbsoluteAnchor) {
            return ((CTAbsoluteAnchor)cellAnchor).getPos(true);
        }
        log.warn("xlsx export:getPos... wrong anchor type used");
        return new CTPoint2D();
    }

    public static ICellAnchor setDrawingAnchor(JSONObject drawingProperties, ICellAnchor sourceAnchor) throws JSONException {

        ICellAnchor cellAnchor = sourceAnchor;
        String anchorType = drawingProperties.optString(OCKey.ANCHOR_TYPE.value(), null);

        // check if we have to switch to another anchorType
        if(anchorType!=null) {
            boolean isTwoCell = anchorType.equals("twoCell")||anchorType.equals("twoCellAsOneCell")||anchorType.equals("twoCellAsAbsolute");
            if(isTwoCell) {
                if(!(sourceAnchor instanceof CTTwoCellAnchor)) {
                    cellAnchor = Context.getDmlSpreadsheetDrawingObjectFactory().createCTTwoCellAnchor();
                    if(sourceAnchor instanceof CTOneCellAnchor) {
                        ((CTTwoCellAnchor)cellAnchor).setFrom(((CTOneCellAnchor)sourceAnchor).getFrom(false));
                    }
                }
                if(anchorType.equals("twoCell")) {
                    ((CTTwoCellAnchor)cellAnchor).setEditAs(org.docx4j.dml.spreadsheetdrawing.STEditAs.TWO_CELL);
                }
                else if(anchorType.equals("twoCellAsOneCell")) {
                    ((CTTwoCellAnchor)cellAnchor).setEditAs(org.docx4j.dml.spreadsheetdrawing.STEditAs.ONE_CELL);
                }
                else if(anchorType.equals("twoCellAsAbsolute")) {
                    ((CTTwoCellAnchor)cellAnchor).setEditAs(org.docx4j.dml.spreadsheetdrawing.STEditAs.ABSOLUTE);
                }
            }
            else if(anchorType.equals("oneCell")&&!(sourceAnchor instanceof CTOneCellAnchor)) {
                cellAnchor = Context.getDmlSpreadsheetDrawingObjectFactory().createCTOneCellAnchor();
                if(sourceAnchor instanceof CTTwoCellAnchor) {
                    ((CTOneCellAnchor)cellAnchor).setFrom(((CTTwoCellAnchor)sourceAnchor).getFrom(false));
                }
                else {
                    ((CTOneCellAnchor)cellAnchor).setExt(((CTAbsoluteAnchor)sourceAnchor).getExt(false));
                }
            }
            else if(anchorType.equals("absolute")&&!(sourceAnchor instanceof CTAbsoluteAnchor)) {
                cellAnchor = Context.getDmlSpreadsheetDrawingObjectFactory().createCTAbsoluteAnchor();
                if(sourceAnchor instanceof CTOneCellAnchor) {
                    ((CTAbsoluteAnchor)cellAnchor).setExt(((CTOneCellAnchor)sourceAnchor).getExt(false));
                }
            }
            if(cellAnchor!=sourceAnchor) {
                cellAnchor.setClientData(sourceAnchor.getClientData());
                cellAnchor.setCxnSp(sourceAnchor.getCxnSp());
                cellAnchor.setGraphicFrame(sourceAnchor.getGraphicFrame());
                cellAnchor.setGrpSp(sourceAnchor.getGrpSp());
                cellAnchor.setPic(sourceAnchor.getPic());
                cellAnchor.setSp(sourceAnchor.getSp());
                cellAnchor.setAlternateContent(sourceAnchor.getAlternateContent());
            }
        }

        final Iterator<String> drawingKeys = drawingProperties.keys();
        while(drawingKeys.hasNext()) {
            final String key = drawingKeys.next();
            final Object value = drawingProperties.get(key);
            final Integer intValue = value instanceof Integer ? (Integer)value : null;

            if(key.equals(OCKey.START_COL.value())&&intValue!=null){
                getFrom(cellAnchor).setCol(intValue);
            }
            else if(key.equals(OCKey.START_COL_OFFSET.value())&&intValue!=null) {
                getFrom(cellAnchor).setColOff(Commons.coordinateFrom100TH_MM(intValue));
            }
            else if(key.equals(OCKey.START_ROW.value())&&intValue!=null) {
                getFrom(cellAnchor).setRow(intValue);
            }
            else if(key.equals(OCKey.START_ROW_OFFSET.value())&&intValue!=null) {
                getFrom(cellAnchor).setRowOff(Commons.coordinateFrom100TH_MM(intValue));
            }
            else if(key.equals(OCKey.END_COL.value())&&intValue!=null) {
                getTo(cellAnchor).setCol(intValue);
            }
            else if(key.equals(OCKey.END_COL_OFFSET.value())&&intValue!=null) {
                getTo(cellAnchor).setColOff(Commons.coordinateFrom100TH_MM(intValue));
            }
            else if(key.equals(OCKey.END_ROW.value())&&intValue!=null) {
                getTo(cellAnchor).setRow(intValue);
            }
            else if(key.equals(OCKey.END_ROW_OFFSET.value())&&intValue!=null)  {
                getTo(cellAnchor).setRowOff(Commons.coordinateFrom100TH_MM(intValue));
            }
            else if(key.equals(OCKey.LEFT.value())&&intValue!=null) {
                getPos(cellAnchor).setX(Commons.coordinateFrom100TH_MM(intValue));
            }
            else if(key.equals(OCKey.TOP.value())&&intValue!=null) {
                getPos(cellAnchor).setY(Commons.coordinateFrom100TH_MM(intValue));
            }
            else if(key.equals(OCKey.WIDTH.value())&&intValue!=null) {
                getExt(cellAnchor).setCx(Commons.coordinateFrom100TH_MM(intValue));
            }
            else if(key.equals(OCKey.HEIGHT.value())&&intValue!=null) {
                getExt(cellAnchor).setCy(Commons.coordinateFrom100TH_MM(intValue));
            }
        }
        return cellAnchor;
    }

    public static ChartWrapper getChart(Map<String, Object> drawingPropertyMap){
        return (ChartWrapper) drawingPropertyMap.get(OCKey.CHART_PART.value());
    }

    public static CTShape createShape() {
        return createDummyShape();
    }

    public static CTGroupShape createGroupShape() {
        final CTGroupShape shape = Context.getDmlSpreadsheetDrawingObjectFactory().createCTGroupShape();

        // general shape Properties
        final CTGroupShapeProperties shapeProperties = Context.getDmlObjectFactory().createCTGroupShapeProperties();
        shape.setGrpSpPr(shapeProperties);
        DMLHelper.createTransform2D(shape.getXfrm(true), 0, 0, 500, 500);
        shape.setNvGrpSpPr(createGrpShapeNonVisual("shape"));
        return shape;
    }

    public static CTConnector createConnectorShape() {
        final ObjectFactory pmlObjectFactory = Context.getDmlSpreadsheetDrawingObjectFactory();
        final org.docx4j.dml.ObjectFactory dmlObjectFactory = Context.getDmlObjectFactory();
        final CTConnector cxnSp = pmlObjectFactory.createCTConnector();
        cxnSp.setNvCxnSpPr(createConnectorShapeNonVisual("connector"));
        cxnSp.setSpPr(dmlObjectFactory.createCTShapeProperties());
        DMLHelper.createTransform2D(cxnSp.getSpPr().getXfrm(true), 0, 0, 500, 500);
        return cxnSp;
    }

    public static CTPicture createImage() {
        final org.docx4j.dml.ObjectFactory dmlObjectFactory = Context.getDmlObjectFactory();

        final CTPicture pic = Context.getDmlSpreadsheetDrawingObjectFactory().createCTPicture();
        final CTNonVisualDrawingProps nvPr = pic.getNonVisualDrawingProperties(true);
        final CTNonVisualPictureProperties nvPicPr = pic.getNonVisualDrawingShapeProperties(true);
        nvPicPr.getLocks(true).setNoChangeAspect(Boolean.TRUE);

        final CTShapeProperties spPr = dmlObjectFactory.createCTShapeProperties();
        final CTPresetGeometry2D presetGeometry = dmlObjectFactory.createCTPresetGeometry2D();
        presetGeometry.setPrst(STShapeType.RECT);
        presetGeometry.setAvLst(dmlObjectFactory.createCTGeomGuideList());
        spPr.setPrstGeom(presetGeometry);
        pic.setSpPr(spPr);
        final CTBlipFillProperties blipFillProperties = pic.getBlipFill(true);
        blipFillProperties.getStretch(true).setFillRect(dmlObjectFactory.createCTRelativeRect());
        return pic;
    }

}
